@@ -88,6 +88,8 @@ static const char *sm_config_prefix_arg_name G_GNUC_UNUSED = "--sm-config-prefix
 static const char *sm_screen G_GNUC_UNUSED = "--screen";
 #endif
 
+extern void _gnome_ui_gettext_init (gboolean bind_codeset) G_GNUC_INTERNAL;
+
 /* The master client.  */
 static GnomeClient *master_client= NULL;
 
@@ -611,7 +613,11 @@ client_save_yourself_callback (SmcConn   smc_conn,
   gboolean ret;
 
   if (!client_grab_widget)
-    client_grab_widget = gtk_widget_new (GTK_TYPE_INVISIBLE, NULL);
+    {
+      GDK_THREADS_ENTER();
+      client_grab_widget = gtk_widget_new (GTK_TYPE_INVISIBLE, NULL);
+      GDK_THREADS_LEAVE();
+    }
 
   /* The first SaveYourself after registering for the first time
    * is a special case (SM specs 7.2).
@@ -689,6 +695,8 @@ client_save_yourself_callback (SmcConn   smc_conn,
 
   client_set_state (client, GNOME_CLIENT_SAVING_PHASE_1);
 
+  GDK_THREADS_ENTER();
+
   if (gdk_pointer_is_grabbed())
     {
       gboolean waiting = TRUE;
@@ -702,12 +710,17 @@ client_save_yourself_callback (SmcConn   smc_conn,
   /* Check that we did not receive a shutdown cancelled while waiting
    * for the grab to be released. The grab should prevent it but ... */
   if (client->state != GNOME_CLIENT_SAVING_PHASE_1)
-    return;
+    {
+      GDK_THREADS_LEAVE();
+      return;
+    }
 
   gdk_pointer_ungrab (GDK_CURRENT_TIME);
   gdk_keyboard_ungrab (GDK_CURRENT_TIME);
   gtk_grab_add (client_grab_widget);
 
+  GDK_THREADS_LEAVE();
+
   name = g_strdup (gnome_client_get_global_config_prefix(client));
   name[strlen (name) - 1] = '\0';
 
@@ -768,7 +781,11 @@ client_die_callback (SmcConn smc_conn, SmPointer client_data)
   GnomeClient *client= (GnomeClient*) client_data;
 
   if (client_grab_widget)
-    gtk_grab_remove (client_grab_widget);
+    {
+      GDK_THREADS_ENTER();
+      gtk_grab_remove (client_grab_widget);
+      GDK_THREADS_LEAVE();
+    }
 
   g_signal_emit (client, client_signals[DIE], 0);
 }
@@ -780,7 +797,11 @@ client_save_complete_callback (SmcConn smc_conn, SmPointer client_data)
   GnomeClient *client = (GnomeClient*) client_data;
 
   if (client_grab_widget)
-    gtk_grab_remove (client_grab_widget);
+    {
+      GDK_THREADS_ENTER();
+      gtk_grab_remove (client_grab_widget);
+      GDK_THREADS_LEAVE();
+    }
 
   g_signal_emit (client, client_signals[SAVE_COMPLETE], 0);
 }
@@ -792,7 +813,11 @@ client_shutdown_cancelled_callback (SmcConn smc_conn, SmPointer client_data)
   GnomeClient *client= (GnomeClient*) client_data;
 
   if (client_grab_widget)
-    gtk_grab_remove (client_grab_widget);
+    {
+      GDK_THREADS_ENTER();
+      gtk_grab_remove (client_grab_widget);
+      GDK_THREADS_LEAVE();
+    }
 
   g_signal_emit (client, client_signals[SHUTDOWN_CANCELLED], 0);
 }
@@ -995,7 +1020,24 @@ gnome_client_module_instance_init (GnomeProgram *program, GnomeModuleInfo *mod_i
 {
     GnomeProgramPrivate_gnome_client *priv = g_new0 (GnomeProgramPrivate_gnome_client, 1);
 
-    g_object_set_qdata (G_OBJECT (program), quark_gnome_program_private_gnome_client, priv);
+    g_object_set_qdata_full (G_OBJECT (program), quark_gnome_program_private_gnome_client, priv, g_free);
+}
+
+static GOptionGroup *
+gnome_client_module_get_goption_group (void)
+{
+	GOptionGroup *option_group;
+
+	_gnome_ui_gettext_init (TRUE);
+
+	option_group = g_option_group_new ("gnome-session",
+					   N_("Session management"),
+					   N_("Show session management options"),
+					   NULL, NULL);
+	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
+	g_option_group_add_entries(option_group, session_goptions);
+
+	return option_group;
 }
 
 const GnomeModuleInfo *
@@ -1010,20 +1052,12 @@ gnome_client_module_info_get (void)
 		NULL, NULL
 	};
 
-	GOptionGroup * option_group;
-	option_group = g_option_group_new ("gnome-session",
-					   N_("Session management"),
-					   N_("Show session management options"),
-					   NULL, NULL);
-	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
-	g_option_group_add_entries(option_group, session_goptions);
-
-	module_info.expansion1 = (gpointer) option_group;
+	module_info.get_goption_group_func = gnome_client_module_get_goption_group;
 
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[3];
 
-		bindtextdomain (GETTEXT_PACKAGE, GNOMEUILOCALEDIR);
+		_gnome_ui_gettext_init (FALSE);
 
 		req[0].required_version = "1.3.7";
 		req[0].module_info = gnome_gtk_module_info_get ();
