@@ -192,12 +192,12 @@ drag_data_get(GnomeHRef          *href,
 		char *s = g_strdup_printf("file:%s\r\n", href->_priv->url);
 		gtk_selection_data_set (selection_data,
 					selection_data->target,
-					8, s, strlen(s)+1);
+					8, (unsigned char *)s, strlen(s)+1);
 		g_free(s);
 	} else {
 		gtk_selection_data_set (selection_data,
 					selection_data->target,
-					8, href->_priv->url, strlen(href->_priv->url)+1);
+					8, (unsigned char *)href->_priv->url, strlen(href->_priv->url)+1);
 	}
 }
 
@@ -391,7 +391,7 @@ gnome_href_clicked (GtkButton *button)
   if(!gnome_url_show_on_screen (href->_priv->url, gtk_widget_get_screen (GTK_WIDGET (href)), NULL)) {
     GtkWidget *dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR,
 					       GTK_BUTTONS_OK,
-					       _("An error has occured while trying to launch the "
+					       _("An error has occurred while trying to launch the "
 						 "default web browser.\n"
 						 "Please check your settings in the "
 						 "'Preferred Applications' preference tool."));
