@@ -21,6 +21,7 @@
 */
 
 #include <config.h>
+#include <libgnome/gnome-macros.h>
 
 #ifndef GNOME_DISABLE_DEPRECATED_SOURCE
 
@@ -46,86 +47,27 @@ struct _GnomeMessageBoxPrivate {
 	int dummy;
 };
 
-static void gnome_message_box_class_init (GnomeMessageBoxClass *klass);
-static void gnome_message_box_init       (GnomeMessageBox      *messagebox);
-static void gnome_message_box_destroy    (GtkObject            *object);
-static void gnome_message_box_finalize   (GObject              *object);
-static void gnome_message_box_construct  (GnomeMessageBox      *messagebox,
-					  const gchar          *message,
-					  const gchar          *message_box_type,
-					  const gchar         **buttons);
-
-static GnomeDialogClass *parent_class;
-
-GType
-gnome_message_box_get_type (void)
-{
-	static GType message_box_type = 0;
-
-	if (!message_box_type)
-	{
-		GtkTypeInfo message_box_info =
-		{
-			"GnomeMessageBox",
-			sizeof (GnomeMessageBox),
-			sizeof (GnomeMessageBoxClass),
-			(GtkClassInitFunc) gnome_message_box_class_init,
-			(GtkObjectInitFunc) gnome_message_box_init,
-			NULL,
-			NULL,
-			NULL
-		};
-
-		message_box_type = gtk_type_unique (GNOME_TYPE_DIALOG, &message_box_info);
-	}
-
-	return message_box_type;
-}
+GNOME_CLASS_BOILERPLATE (GnomeMessageBox, gnome_message_box,
+			 GnomeDialog, GNOME_TYPE_DIALOG)
 
 static void
 gnome_message_box_class_init (GnomeMessageBoxClass *klass)
 {
+	/*
 	GtkObjectClass *object_class;
-	GObjectClass *gobject_class;
 
 	object_class = (GtkObjectClass *)klass;
-	gobject_class = (GObjectClass *)klass;
-	parent_class = gtk_type_class (GNOME_TYPE_DIALOG);
-
-	object_class->destroy = gnome_message_box_destroy;
-	gobject_class->finalize = gnome_message_box_finalize;
+	*/
 }
 
 static void
-gnome_message_box_init (GnomeMessageBox *message_box)
+gnome_message_box_instance_init (GnomeMessageBox *message_box)
 {
 	/*
  	message_box->_priv = g_new0(GnomeMessageBoxPrivate, 1); 
 	*/
 }
 
-static void
-gnome_message_box_destroy(GtkObject *object)
-{
-	/* remember, destroy can be run multiple times! */
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
-static void
-gnome_message_box_finalize(GObject *object)
-{
-	/*
-	GnomeMessageBox *mbox = GNOME_MESSAGE_BOX(object);
-
-	g_free(mbox->_priv);
-	mbox->_priv = NULL;
-	*/
-
-	if (G_OBJECT_CLASS (parent_class)->finalize)
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
-}
-
 /**
  * gnome_message_box_construct:
  * @messagebox: The message box to construct
@@ -135,10 +77,8 @@ gnome_message_box_finalize(GObject *object)
  *
  * For language bindings or subclassing, from C use #gnome_message_box_new or
  * #gnome_message_box_newv
- *
- * Returns:
  */
-static void
+void
 gnome_message_box_construct (GnomeMessageBox       *messagebox,
 			     const gchar           *message,
 			     const gchar           *message_box_type,
@@ -152,13 +92,13 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
 	GtkStyle *style;
         const gchar* title_prefix = NULL;
         const gchar* appname;
-	gint i = 0;
 
 	g_return_if_fail (messagebox != NULL);
 	g_return_if_fail (GNOME_IS_MESSAGE_BOX (messagebox));
 	g_return_if_fail (message != NULL);
 	g_return_if_fail (message_box_type != NULL);
 
+	atk_object_set_role (gtk_widget_get_accessible (GTK_WIDGET (messagebox)), ATK_ROLE_ALERT);
 	style = gtk_widget_get_style (GTK_WIDGET (messagebox));
 
 	/* Make noises, basically */
@@ -197,12 +137,9 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
         if (appname) {
                 s = g_strdup_printf("%s (%s)", title_prefix, appname);
         }
-        if (s) {
-                gtk_window_set_title(GTK_WINDOW(messagebox), s);
-                g_free(s);
-        } else {
-                gtk_window_set_title(GTK_WINDOW(messagebox), title_prefix);
-        }
+
+	gnome_dialog_constructv (GNOME_DIALOG (messagebox), s ? s : title_prefix, buttons);
+	g_free (s);
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(messagebox)->vbox),
@@ -225,23 +162,11 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
 	/* Add some extra space on the right to balance the pixmap */
 	if (pixmap) {
 		alignment = gtk_alignment_new (0., 0., 0., 0.);
-		gtk_widget_set_usize (alignment, GNOME_PAD, -1);
+		gtk_widget_set_size_request (alignment, GNOME_PAD, -1);
 		gtk_widget_show (alignment);
 		
 		gtk_box_pack_start (GTK_BOX (hbox), alignment, FALSE, FALSE, 0);
 	}
-
-	if (buttons) {
-		while (buttons[i]) {
-			gnome_dialog_append_button (GNOME_DIALOG (messagebox), 
-						    buttons[i]);
-			i++;
-		};
-	}
-
-	if(GNOME_DIALOG(messagebox)->buttons)
-		gtk_widget_grab_focus(
-			g_list_last (GNOME_DIALOG (messagebox)->buttons)->data);
 	
 	gnome_dialog_set_close (GNOME_DIALOG (messagebox),
 				TRUE );
@@ -271,7 +196,7 @@ gnome_message_box_new (const gchar           *message,
         
 	va_start (ap, message_box_type);
 	
-	message_box = gtk_type_new (GNOME_TYPE_MESSAGE_BOX);
+	message_box = g_object_new (GNOME_TYPE_MESSAGE_BOX, NULL);
 
 	gnome_message_box_construct (message_box, message,
 				     message_box_type, NULL);
@@ -292,8 +217,10 @@ gnome_message_box_new (const gchar           *message,
 	
 	va_end (ap);
 
-	gtk_widget_grab_focus(
-		g_list_last (GNOME_DIALOG (message_box)->buttons)->data);
+	if (GNOME_DIALOG (message_box)->buttons) {
+		gtk_widget_grab_focus(
+			g_list_last (GNOME_DIALOG (message_box)->buttons)->data);
+	}
 
 	return GTK_WIDGET (message_box);
 }
@@ -321,7 +248,7 @@ gnome_message_box_newv (const gchar           *message,
 	g_return_val_if_fail (message != NULL, NULL);
 	g_return_val_if_fail (message_box_type != NULL, NULL);
 
-	message_box = gtk_type_new (GNOME_TYPE_MESSAGE_BOX);
+	message_box = g_object_new (GNOME_TYPE_MESSAGE_BOX, NULL);
 
 	gnome_message_box_construct (message_box, message,
 				     message_box_type, buttons);
