@@ -95,8 +95,7 @@ static void libgnomeui_post_args_parse	(GnomeProgram *app,
 					 GnomeModuleInfo *mod_info);
 static void libgnomeui_rc_parse		(GnomeProgram *program);
 #ifndef G_OS_WIN32
-static void libgnomeui_segv_setup	(GnomeProgram *program,
-					 gboolean post_arg_parse);
+static void libgnomeui_segv_setup	(GnomeProgram *program);
 #endif
 
 /* Prototype for a private gnome_stock function */
@@ -136,11 +135,37 @@ static GOptionEntry libgnomeui_goptions[] = {
 	{ NULL }
 };
 
+void _gnome_ui_gettext_init (gboolean) G_GNUC_INTERNAL;
+
+void G_GNUC_INTERNAL
+_gnome_ui_gettext_init (gboolean bind_codeset)
+{
+	static gboolean initialized = FALSE;
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET	
+	static gboolean codeset_bound = FALSE;
+#endif
+
+	if (!initialized)
+	{
+		bindtextdomain (GETTEXT_PACKAGE, GNOMEUILOCALEDIR);
+		initialized = TRUE;
+	}
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET	
+	if (!codeset_bound && bind_codeset)
+	{
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+		codeset_bound = TRUE;
+	}
+#endif
+}
+
 static GOptionGroup *
 libgnomeui_get_goption_group (void)
 {
 	GOptionGroup *option_group;
 
+	_gnome_ui_gettext_init (TRUE);
+
 	option_group = g_option_group_new ("gnome-ui",
 					   N_("GNOME GUI Library"),
 					   N_("Show GNOME GUI options"),
@@ -168,7 +193,7 @@ libgnomeui_module_info_get (void)
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[6];
 
-		bindtextdomain (GETTEXT_PACKAGE, GNOMEUILOCALEDIR);
+		_gnome_ui_gettext_init (FALSE);
 
 		req[0].required_version = "1.101.2";
 		req[0].module_info = LIBBONOBOUI_MODULE;
@@ -204,6 +229,14 @@ typedef struct {
 static GQuark quark_gnome_program_private_libgnomeui = 0;
 static GQuark quark_gnome_program_class_libgnomeui = 0;
 
+static void
+libgnomeui_private_free (GnomeProgramPrivate_libgnomeui *priv)
+{
+	g_free (priv->display);
+	g_free (priv->default_icon);
+	g_free (priv);
+}
+
 static void
 libgnomeui_get_property (GObject *object, guint param_id, GValue *value,
                          GParamSpec *pspec)
@@ -305,26 +338,24 @@ libgnomeui_instance_init (GnomeProgram *program, GnomeModuleInfo *mod_info)
 {
     GnomeProgramPrivate_libgnomeui *priv = g_new0 (GnomeProgramPrivate_libgnomeui, 1);
 
-    g_object_set_qdata (G_OBJECT (program), quark_gnome_program_private_libgnomeui, priv);
+    g_object_set_qdata_full (G_OBJECT (program),
+			     quark_gnome_program_private_libgnomeui,
+			     priv, (GDestroyNotify) libgnomeui_private_free);
 }
 
 static void
 libgnomeui_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 {
-        gboolean do_crash_dialog = TRUE;
         const char *envar;
 
+        /* This is one of two places where the crash dialog can be disabled */
+        /* The other is the command line option --disable-crash-dialog, so */
+        /* we wait until post_args_parse before calling libgnomeui_segv_setup */
         envar = g_getenv("GNOME_DISABLE_CRASH_DIALOG");
-        if(envar)
-                do_crash_dialog = atoi(envar)?FALSE:TRUE;
-	if ( ! do_crash_dialog)
+        if (envar && atoi (envar))
 		g_object_set (G_OBJECT(app),
 			      LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
 			      NULL);
-#ifndef G_OS_WIN32
-        if(do_crash_dialog)
-                libgnomeui_segv_setup (app, FALSE);
-#endif
 }
 
 #ifdef HAVE_ESD
@@ -573,7 +604,7 @@ libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
         gnome_type_init();
         libgnomeui_rc_parse (program);
 #ifndef G_OS_WIN32
-        libgnomeui_segv_setup (program, TRUE);
+        libgnomeui_segv_setup (program);
 #endif
         priv = g_object_get_qdata(G_OBJECT(program), quark_gnome_program_private_libgnomeui);
         priv->constructed = TRUE;
@@ -585,10 +616,7 @@ libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
         gtk_accel_map_load (filename);
         g_free (filename);
 
-	bindtextdomain (GETTEXT_PACKAGE, GNOMEUILOCALEDIR);
-#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif
+	_gnome_ui_gettext_init (TRUE);
 
         _gnome_stock_icons_init ();
 
@@ -714,7 +742,7 @@ libgnomeui_rc_parse (GnomeProgram *program)
 static void libgnomeui_segv_handle(int signum);
 
 static void
-libgnomeui_segv_setup (GnomeProgram *program, gboolean post_arg_parse)
+libgnomeui_segv_setup (GnomeProgram *program)
 {
         static struct sigaction *setptr;
         struct sigaction sa;
